# Converts a SCI vocab.000 file to a spreadsheet & back
#
# Usage:     perl vocabEditor.pl inputfile outputfile
#            Where the input file is the file to be read from & the output file is the file to be written to
#
# Examples:  perl vocabEditor.pl vocab.000 outputspreadsheet.xls
#            perl vocabEditor.pl outputspreadsheet.xls vocabnew.000
#
# Notes:     -  Spreadsheet is alphabeticlly sorted prior to exporting to vocab.000 format (thereby taking advantange of header indexes)
#
# To Do:     -  Dissect the word class to individual bits & map as separate columns in the spreadsheet
#            -  Create an indicator as to what the next available group number is
#            -  Display association between words within the same group
#
# Do whatever you want with this file, except remove my name.
# Jeremiah Nellis, nellisjp@hotmail.com
# Version 1.0 - Sep 17th 2010

use strict;
use warnings;

use Spreadsheet::ParseExcel;
use Spreadsheet::WriteExcel;

my $file;
my $data;
my $hex;	

my $rest;
my $hexVal;
my $hexVal1;
my $hexVal2;
my $word;
my $wordClass;

my $wordClass1;
my $wordClass2;
my $wordClass3;
my $group1;
my $group2;
my $group3;

my $group;
my $lastword;
my $index = 0;

my $header;
my $buffer;
my $lastLetter;
my $row = 0;
my $i = 0;
my @vocab;


if (@ARGV < 2) { 
      print "Usage: vocabEditor.exe %1 %2\n";
      print "  %1: input file\n";
	  print "  %2: output file\n";
	  print "    Basic 'from-to' arguments.  From vocab-to-spreadsheet or spreadsheet-to-vocab\n";
      exit (1);
}

my $inputFile = $ARGV[0];
my $outputFile = $ARGV[1];

if ($inputFile =~ m/\.000/) {
   print "Converting vocab file to spreadsheet...";
   goto ToSpreadsheet;
} else {
   print "Converting spreadsheet to vocab file...";
   goto ToVocab;
}

print "Bad first arg, aborting.\n";
exit(0);

ToSpreadsheet:
# Create a new Excel workbook
my $workbook = Spreadsheet::WriteExcel->new($outputFile);

# Add a worksheet
my $worksheet = $workbook->add_worksheet();
my $format1 = $workbook->add_format(num_format => '@');

# Read whole file in
undef $/;
open FILE, $inputFile or die "Can't open $file $!\n";
# binmode FILE to supress conversion of line endings
binmode FILE;
$data = <FILE>;
close FILE;
# convert data to hex form
$hex = unpack 'H*', $data;
$rest = $hex;

# Read the first 54 characters, skipping the header
while ($index < 55) {
   ($header, $rest) = split //, $rest, 2;
   ($header, $rest) = split //, $rest, 2;
   $index++;
}

while ($rest) {
    ($hexVal1, $rest) = split //, $rest, 2;
	($hexVal2, $rest) = split //, $rest, 2;
	$hexVal = hex($hexVal1 . $hexVal2);

	    
	   # Any character above 128 indicates the end of a word, subtract 128 for the final letter
	   if ($hexVal - 128 > 0) {
		   $hexVal = chr($hexVal - int(128));
		   $word = $word . $hexVal;
		   
		   ($wordClass1, $rest) = split //, $rest, 2;
	       ($wordClass2, $rest) = split //, $rest, 2;
	       ($wordClass3, $rest) = split //, $rest, 2;
	   
	       ($group1, $rest) = split //, $rest, 2;
	       ($group2, $rest) = split //, $rest, 2;
	       ($group3, $rest) = split //, $rest, 2;

		   # print "WriteExcel: Word = " . $word . ", Wordclass = " . $wordClass1 . $wordClass2 . $wordClass3 . ", Group = " . $group1 . $group2 . $group3 . "\n";
		   
		   $worksheet->write_string($row, 0, $word, $format1);
		   $worksheet->write_string($row, 1, $wordClass1 . $wordClass2 . $wordClass3, $format1);
		   $worksheet->write_string($row, 2, $group1 . $group2 . $group3, $format1);
	
		   # Hold the last word so we can use the starting characters for the next word;
		   $lastword = $word;

		   # Read the next hex, convert to an integer and use that many characters from the start of the last word
		   (my $charCount1, $rest) = split //, $rest, 2;
		   unless (defined $rest) { last;}
		   (my $charCount2, $rest) = split //, $rest, 2;
		   	   
		   my $charCount = int($charCount1 . $charCount2);
		   #print "Copying " . $charCount . " characters from the last word\n";
		   if ($charCount > 0) {
		      $word = substr($lastword, 0, $charCount);
		   } else {
              $word = undef;
           } 		   
          
		   $row++;
		   
       } else {      
           $word = $word . chr($hexVal);
       }
	   

}

#  End of converting vocab-to-spreadsheet
print "Done!\n";
exit(0);

#
#  Begin of converting spreadsheet-to-vocab
#
ToVocab:

#
#  Read an existing spreadsheet & convert it to vocab.000 format
#

my $oExcel = new Spreadsheet::ParseExcel;
my $oBook;
my $oWkC;
my $oWkS;

$oBook = $oExcel->Parse($inputFile);

# Load the spreadsheet into an array for sorting

    $i=0;
    for(my $iSheet=0; $iSheet < $oBook->{SheetCount} ; $iSheet++) {
        $oWkS = $oBook->{Worksheet}[$iSheet];
        for(my $row = $oWkS->{MinRow}; defined $oWkS->{MaxRow} && $row <= $oWkS->{MaxRow} ; $row++) {
			  
			$oWkC = $oWkS->{Cells}[$row][0];
            $word = $oWkC->{Val}; 			
			
            $oWkC = $oWkS->{Cells}[$row][1];
            $wordClass = $oWkC->{Val};

            $oWkC = $oWkS->{Cells}[$row][2];
            $group = $oWkC->{Val};
			
			unless (defined $word) {
			   next;
			}
             
            push @vocab, { word => $word, wordClass => $wordClass, group => $group };
            $i++;			
		}
	}
	
# Sort the array
my @sortedVocab =  sort { $a->{word} cmp $b->{word} } @vocab;			 

open FILE, ">" . $outputFile or die "Can't open $outputFile $!\n";
# binmode FILE to supress conversion of line endings
binmode FILE;

	# Start the index at the beginning of the body
	$index = 52;
	
	$header = pack "H*", sprintf("%x",134);
	$lastword = undef;
	$lastLetter = 97; # set to 'a';
	
    for(my $row=0; $row < $i; $row++) {
			  
			$word = $sortedVocab[$row]{'word'};
			$wordClass = $sortedVocab[$row]{'wordClass'};
			$group = $sortedVocab[$row]{'group'};
			
			unless (defined $word) {
			   last;
			}
			
            # print "ExcelParse: Word = " . $word . ", WordClass = " . $wordClass . ", Group = " . $group . " Index = " . $index;			
            my($matchLen) = &matchLen($word, $lastword); 			
			
			# If this is a new word & is alpha (a-z), save the current location in hex for the header;
			if ($matchLen == 0 && ord(substr($word,0,1)) >= 97 && ord(substr($word,0,1)) <= 122) {

			   $header = $header . pack "H*", sprintf("%04x",$index);
 
			   while(ord(substr($word,0,1)) - $lastLetter > 1) {
			    # print ord(substr($word,0,1)) . "\n";
			    # print $lastLetter .  "\n";

                $header = $header . pack "H*", "0000";
			    $lastLetter++;
			   }
			   $lastLetter = ord(substr($word,0,1));
			}
			
	        # Output the count of the matchlen (in hex)
			print FILE chr($matchLen);
			$index++;
			
			print FILE substr($word,$matchLen,length($word) - $matchLen - 1);
            $index = $index + length( substr($word,$matchLen,length($word) - $matchLen - 1) );
            
            # Print the last character of the word with an offset of +128
            print FILE chr ( ord( substr($word, length($word) - 1, 1) ) + int(128) );
			$index++;

			# Write out the word class and group
			print FILE pack 'H*', substr($wordClass, 0, 2);
			print FILE pack 'H*', substr($wordClass, 2, 1) . substr($group, 0, 1);
		    print FILE pack 'H*', substr($group, 1, 2);
			$index = $index + 3;						
			$lastword = $word;
    }
	
close FILE;

# Reopen file, read it into the buffer, append the header & write it out


open(FILE,$outputFile) || die;
binmode FILE;
while(<FILE>) { $buffer .= $_; }
close(FILE);

# Terminate the header (any char will do!) 
$header = $header . pack "H*", "0";
$buffer = $header . $buffer;

open(FILE,">" . $outputFile) || die;
binmode FILE;
print FILE $buffer;
close(FILE);

print "Done!\n";
exit(0);

	
sub matchLen ($$) {
   my($inputStr, $matchStr) = @_;
   my $i=0;

   if(!(defined $matchStr) || !(defined $inputStr)) {
     return(0);
   }
   while(substr($inputStr,$i,1) eq substr($matchStr,$i,1)) {
      $i++;
	  
	  if ($i == length($inputStr) || $i == length($matchStr)) {
	     last;
	  }
   }
   return $i;
}


